#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec3 color_world;
	vec2 uv_world;
} vert;

uniform vec3 camera_world;

// Texture
uniform bool hasTexture;
uniform sampler2D textureSampler;

// SpecMap
uniform bool hasSpecMap;
uniform sampler2D specMapSampler;

// Lights

#define MAX_LIGHTS 10

uniform int dl_count;

uniform vec3 dl_direction[MAX_LIGHTS];
uniform vec3 dl_color[MAX_LIGHTS];

uniform int pl_count;

uniform vec3 pl_position[MAX_LIGHTS];
uniform vec3 pl_color[MAX_LIGHTS];
uniform vec3 pl_attenuation[MAX_LIGHTS];

uniform int sl_count;

uniform vec3 sl_position[MAX_LIGHTS];
uniform vec3 sl_direction[MAX_LIGHTS];
uniform vec3 sl_color[MAX_LIGHTS];

uniform float sl_inner_angle[MAX_LIGHTS];
uniform float sl_outer_angle[MAX_LIGHTS];
uniform vec3 sl_attenuation[MAX_LIGHTS];

// Color & Material

vec3 ambientColor = vec3(1,1,1);
uniform vec3 diffuseColor;

uniform float ambientReflection;
uniform float diffuseReflection;
uniform float specularReflection;
uniform float shininess;

out vec4 color_out;

void main() {	

	vec3 normal_world = normalize(vert.normal_world);

	vec3 to_viewer_vec = normalize(camera_world-vert.position_world);
	
	vec3 objectColor = diffuseColor;
	vec3 textureColor = vec3(0,0,0);
	if(hasTexture) {
		//vec3(vert.uv_world.x,0,vert.uv_world.y);//
		textureColor = texture(textureSampler, vert.uv_world).rgb;
		objectColor = textureColor+objectColor;
	}
	
	float specMapIntensity = specularReflection;
	if(hasSpecMap) {

		specMapIntensity = texture(specMapSampler, vert.uv_world).x;
	}
	
	vec3 color;
	
	color = vec3(0,0,0);
	// ambient
	//color += objectColor*ambientReflection*ambientColor;
	color += objectColor*ambientColor;
	
	color_out = vec4(color, 1);

}