#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 boneid;
layout(location = 4) in vec4 boneweight;

// Bones
#define MAX_BONES 50
#define MAX_WEIGHTS 4

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec3 color_world;
	vec2 uv_world;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;

uniform bool hasBones;
uniform mat4 boneTransforms[MAX_BONES];

void main() {

	vert.color_world = vec3(0, 0, 0);
	
	vec4 totalLocalPosition = vec4(position, 1);
	vec4 totalNormal = vec4(normal, 1);
	
	if(hasBones) {
		totalLocalPosition = vec4(0.0);
		totalNormal = vec4(0.0);
	
	
		for(int i=0;i<MAX_WEIGHTS;i++) {
			mat4 boneTransform = boneTransforms[boneid[i]];
			
			vec4 posePosition = boneTransform * vec4(position, 1.0);
			totalLocalPosition += posePosition * boneweight[i];

			vec4 worldNormal = boneTransform * vec4(normal, 0.0);
			totalNormal += worldNormal * boneweight[i];
		}
		
		mat4 bonetransform;
		bonetransform = boneTransforms[boneid[0]] * boneweight[0];
		bonetransform += boneTransforms[boneid[1]] * boneweight[1];
		bonetransform += boneTransforms[boneid[2]] * boneweight[2];
		bonetransform += boneTransforms[boneid[3]] * boneweight[3];
		
		totalLocalPosition = bonetransform * vec4(position, 1.0);
		totalNormal = bonetransform * vec4(normal, 0.0);
	}

	gl_Position = viewProjMatrix * modelMatrix * vec4(totalLocalPosition);
	vert.position_world = vec3(modelMatrix * vec4(totalLocalPosition));

	vert.normal_world = vec3(transpose(inverse(modelMatrix)) * vec4(totalNormal)); // normals fix
	vert.normal_world = normalize(vert.normal_world);
	
	vert.uv_world = uv;
}
	
	
	
	