#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

layout(location = 3) in ivec4 boneid;
layout(location = 4) in vec4 boneweight;

// Bones
#define MAX_BONES 50
#define MAX_WEIGHTS 4

out VertexData {
	vec3 position_world;
} vert;

uniform mat4 modelMatrix;
uniform mat4 modelScaleMatrix;
uniform mat4 viewProjMatrix;

uniform bool hasBones;
uniform mat4 boneTransforms[MAX_BONES];

void main() {

	vec4 totalLocalPosition = vec4(position, 1);
	vec4 totalNormal = vec4(normal, 1);
	
	if(hasBones) {
		mat4 bonetransform;
		bonetransform = boneTransforms[boneid[0]] * boneweight[0];
		bonetransform += boneTransforms[boneid[1]] * boneweight[1];
		bonetransform += boneTransforms[boneid[2]] * boneweight[2];
		bonetransform += boneTransforms[boneid[3]] * boneweight[3];
		
		totalLocalPosition = bonetransform * vec4(position, 1.0);
		totalNormal = bonetransform * vec4(normal, 0.0);
	}

	vec4 pos = vec4(vec3(totalLocalPosition.x, totalLocalPosition.y, totalLocalPosition.z)
	/*+ normalize(normal)*0.05*/,1.0);
	
	/*vec4 modelPos = modelMatrix * pos;
	vec4 modelScalePos = modelScaleMatrix * pos;
	
	vec3 modelPos3 = vec3(modelPos.xyz);
	vec3 modelScalePos3 = vec3(modelScalePos.xyz);*/
	
	pos = vec4(vec3(totalLocalPosition.xyz),1.0);
	
	gl_Position = viewProjMatrix * modelMatrix * pos;
}