#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 boneid;
layout(location = 4) in vec4 boneweight;

// Bones
#define MAX_BONES 50
#define MAX_WEIGHTS 4

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec3 color_world;
	vec2 uv_world;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;

uniform bool hasBones;
uniform mat4 boneTransforms[MAX_BONES];

void main() {

	vert.color_world = vec3(0, 0, 0);
	
	vec4 totalLocalPosition = vec4(position, 1);
	vec4 totalNormal = vec4(normal, 1);
	
	if(hasBones) {
		totalLocalPosition = vec4(0.0);
		totalNormal = vec4(0.0);
	
	
		for(int i=0;i<MAX_WEIGHTS;i++) {
			mat4 boneTransform = boneTransforms[boneid[i]];
			
			vec4 posePosition = boneTransform * vec4(position, 1.0);
			totalLocalPosition += posePosition * boneweight[i];

			vec4 worldNormal = boneTransform * vec4(normal, 0.0);
			totalNormal += worldNormal * boneweight[i];
		}
	

/*
		mat4 trans1 = boneTransforms[boneid[0]];
		mat4 trans2 = boneTransforms[boneid[1]];
		mat4 trans3 = boneTransforms[boneid[2]];
		mat4 trans4 = boneTransforms[boneid[3]];
		totalLocalPosition = trans4 * vec4(position, 1.0)* boneweight[0]+
							trans3 * vec4(position, 1.0)* boneweight[1]+
							trans2 * vec4(position, 1.0)* boneweight[2]+
							trans1 * vec4(position, 1.0)* boneweight[3];
							
		totalNormal = trans4 * vec4(normal, 1.0)* boneweight[0]+
						trans3 * vec4(normal, 1.0)* boneweight[1]+
						trans2 * vec4(normal, 1.0)* boneweight[2]+
						trans1 * vec4(normal, 1.0)* boneweight[3];
					*/
		
		mat4 bonetransform;
		bonetransform = boneTransforms[boneid[0]] * boneweight[0];
		bonetransform += boneTransforms[boneid[1]] * boneweight[1];
		bonetransform += boneTransforms[boneid[2]] * boneweight[2];
		bonetransform += boneTransforms[boneid[3]] * boneweight[3];
		
		totalLocalPosition = bonetransform * vec4(position, 1.0);
		totalNormal = bonetransform * vec4(normal, 0.0);
		
		
		/*
		float primary = 1;
		float max = 0.0;
		for(int i=0;i<4;i++) {
			if(boneweight[i]>max) {
				primary = float(boneid[i]);
				max = boneweight[i];
			}
		}
		*/
		
		//primary = float(boneid[0])+1;
		/*
		primary = primary-1;
		
		if(primary < 4) {
			vert.color_world = vec3(primary/16, 0, 0);
		} else if(primary < 8) {
			vert.color_world = vec3(0, primary/20, 0);
		} else if(primary < 12) {
			vert.color_world = vec3(primary/16, primary/16, 0);
		} else if(primary < 16) {
			vert.color_world = vec3(0, primary/16, primary/16);
		} else {
			vert.color_world = vec3(0, 0, primary/16);
		}
		*/
		//float factor = 16.0;
		//vert.color_world = vec3(primary/factor, 0, 0);
	}

	gl_Position = viewProjMatrix * modelMatrix * vec4(totalLocalPosition);
	vert.position_world = vec3(modelMatrix * vec4(totalLocalPosition));

	vert.normal_world = vec3(transpose(inverse(modelMatrix)) * vec4(totalNormal)); // normals fix
	vert.normal_world = normalize(vert.normal_world);
	
	vert.uv_world = uv;
}
	
	
	
	