#version 430 core
// http://www.mbsoftworks.sk/tutorials/opengl3/23-particle-system/

layout (points) in;
layout (points) out;
layout (max_vertices = 30) out;

//from vertex Shader
in vec3 positionPass[];
in vec3 velocityPass[];
in vec3 colorPass[];
in float lifeTimePass[];
in float sizePass[];
in int typePass[];

//send further
out vec3 positionOut;
out vec3 velocityOut;
out vec3 colorOut;
out float lifeTimeOut;
out float sizeOut;
out int typeOut;

out vec4 colorPart;

uniform vec3 spawnPosition;
uniform vec3 gravity;
uniform vec3 velocityMin; 
uniform vec3 velocityRange;
uniform vec3 color;
uniform float size; 
uniform float lifeTimeMin, lifeTimeRange; 
uniform float timePassed; 
uniform vec3 randomSeed; 
vec3 localSeed;
uniform int numOfParticles; // = numtoGenerate

uniform vec3 positionRange;

uniform mat4 mvp;

// This function returns random number from zero to one
float randZeroOne() {
	uint n = floatBitsToUint(localSeed.y * 214013.0 + localSeed.x * 2531011.0 + localSeed.z * 141251.0);
	n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u;
 
    float fRes =  2.0 - uintBitsToFloat(n);
    localSeed = vec3(localSeed.x + 147158.0 * fRes, localSeed.y*fRes  + 415161.0 * fRes, localSeed.z + 324154.0*fRes);
    return fRes;
}


void main () {


/*
colorOut = colorPass[0];
gl_Position = mvp*vec4(positionPass[0],1);
EmitVertex();
EndPrimitive(); 
*/


localSeed = randomSeed;

positionOut = positionPass[0];
velocityOut = velocityPass[0];

if(typePass[0] != 0) {
	positionOut+=velocityOut*timePassed;
	//positionOut+=vec3(0,0.1,0);
}
if(typePass[0] != 0) {
	velocityOut+=gravity*timePassed;
}


colorOut = colorPass[0];
lifeTimeOut = lifeTimePass[0]-timePassed;
sizeOut = sizePass[0];
typeOut = typePass[0];


/*
gl_Position = mvp*vec4(positionPass[0],1);

EmitVertex();
EndPrimitive(); 
*/

	if (typeOut == 0)  {
		// keep the spawn particle
		EmitVertex(); 
		EndPrimitive();
		for (int i = 0; i< numOfParticles; i++) {
			
			//positionOut = spawnPosition;
			positionOut = spawnPosition+vec3(positionRange.x*randZeroOne(), positionRange.y*randZeroOne(), positionRange.z*randZeroOne());
			velocityOut = velocityMin+vec3(velocityRange.x*randZeroOne(), velocityRange.y*randZeroOne(), velocityRange.z*randZeroOne());
			colorOut = color;
			
			lifeTimeOut = lifeTimeMin+lifeTimeRange*randZeroOne();
			sizeOut = size;
			typeOut = 1;
			
			// spawn normal particles
			gl_Position = mvp*vec4(positionOut,1);
			EmitVertex();
			EndPrimitive();
		}

	} else if(lifeTimeOut > 0.0) {
		// update normal particles
		gl_Position = mvp*vec4(positionOut,1);
		EmitVertex();
		EndPrimitive(); 
	}
  
}

